﻿using Newtonsoft.Json;
using System;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace IUL.Facade
{
  public static class IULFacade
  {
    #region - public methods -

    #region [Get]
    public static T Get<T>(string Exercise)
    {
      var Req = Get(Exercise);
      Req.Wait();
      var ReqJson = Req.Result.Content.ReadAsStringAsync().Result;
      return JsonConvert.DeserializeObject<T>(ReqJson);
    }

    public async static Task<HttpResponseMessage> Get(string Exercise)
    {
      var HttpClient = OpenHttpClient();
      var HttpReq = new HttpRequestMessage(HttpMethod.Get, string.Format(Config.EXERCISE_TEMPLATE, Exercise, "data"));
      var Data = await HttpClient.SendAsync(HttpReq);
      HttpClient.Dispose();
      return Data;
    }
    #endregion

    #region [Send]
    public static HttpResponseMessage Send<T>(string Exercise, T Content)
    {
      var ResJson = JsonConvert.SerializeObject(Content);
      var Res = Send(Exercise, ResJson);
      Res.Wait();
      return Res.Result;
    }

    public async static Task<HttpResponseMessage> Send(string Exercise, string Content)
    {
      var HttpClient = OpenHttpClient();
      var Utf8Content = new StringContent(Content.ToLower(), Encoding.UTF8, "application/json");
      var Data = await HttpClient.PostAsync(string.Format(Config.EXERCISE_TEMPLATE, Exercise, "result"), Utf8Content);
      HttpClient.Dispose();
      return Data;
    }
    #endregion

    #endregion

    #region - private methods -

    private static HttpClient OpenHttpClient()
    {
      var httpClient = new HttpClient();
      var Login = new AuthenticationHeaderValue("Token", Config.TOKEN);
      httpClient.DefaultRequestHeaders.Authorization = Login;
      httpClient.DefaultRequestHeaders.UserAgent.Add(new ProductInfoHeaderValue("Mozilla", "5.0"));
      httpClient.BaseAddress = new Uri(Config.API_BASE_URL);
      return httpClient;
    }

    #endregion
  }
}
